from Products.CMFCore.utils import getToolByName
from Products.CMFFormController.FormAction import FormActionKey
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.Archetypes.public import listTypes

from StringIO import StringIO
    
from Products.PloneJobBoard.config import PROJECTNAME, product_globals
    
# Get the custom workflows
from Products.PloneJobBoard.Extensions import JobBoardWorkflow
from Products.PloneJobBoard.Extensions import JobPostingWorkflow
from Products.PloneJobBoard.Extensions import JobResponseWorkflow

def setWorkflowForType(self, out, tool, metaType, workflowName):
    """Set the workflow for a given type"""
    
    tool.setChainForPortalTypes((metaType,), workflowName)
    
    print >> out, "Set workflow for %s to %s" % (metaType, workflowName,)
     
def addWorkflow(self, out, tool, workflowName, workflowId):
    """Add the givne workflow. The workflowName must be the exact name set
    in the workflow, which in turn must have been registered with the
    workflows factory.
    """
    
    if not workflowId in tool.objectIds():
        tool.manage_addWorkflow(workflowName, workflowId)
        print >> out, "Added workflow", workflowName 
    
def addWorkflowScript(self, out, tool, wfname, id, title, module, method):
    """Add the given external method to the given workflow as a script"""
    
    wf = tool[wfname]
    
    if not id in wf.scripts.objectIds():
        factory = wf.scripts.manage_addProduct['ExternalMethod']
        factory.manage_addExternalMethod(id, title, module, method)

        print >> out, "Added external method %s.%s to workflow %s" % \
                          (module, method, wfname)


def installWorkflows(self, out):

    workflow = getToolByName(self, 'portal_workflow')

    print >> out, "Installing job posting and job response workflows"
    
    addWorkflow(self, out, workflow,   
                    'job_board_workflow (Job board workflow)',
                    'job_board_workflow')
    
    addWorkflow(self, out, workflow,   
                    'job_posting_workflow (Job posting workflow)',
                    'job_posting_workflow')

    addWorkflow(self, out, workflow,   
                    'job_response_workflow (Job response workflow)',
                    'job_response_workflow')
    
    workflow.updateRoleMappings()
    
    # Set up the scripts
    addWorkflowScript(self, out, workflow, 'job_posting_workflow',
                        id = 'post_job_posting',
                        title = 'Finalise posting of job posting',
                        module = 'PloneJobBoard.JobBoardWorkflowScripts',
                        method = 'postJobPosting')
    addWorkflowScript(self, out, workflow, 'job_posting_workflow',
                        id = 'retract_job_posting',
                        title = 'Finalise retraction of job posting',
                        module = 'PloneJobBoard.JobBoardWorkflowScripts',
                        method = 'retractJobPosting')
    addWorkflowScript(self, out, workflow, 'job_response_workflow',
                        id = 'post_job_response',
                        title = 'Finalise posting of job response',
                        module = 'PloneJobBoard.JobBoardWorkflowScripts',
                        method = 'postJobResponse')
    
    # Enable the workflow for the types
    setWorkflowForType(self, out, workflow, 'JobCategory', 'job_board_workflow')
    setWorkflowForType(self, out, workflow, 'JobSubCategory', 'job_board_workflow')
    setWorkflowForType(self, out, workflow, 'JobBoard', 'job_board_workflow')
    setWorkflowForType(self, out, workflow, 'JobPosting', 'job_posting_workflow')
    setWorkflowForType(self, out, workflow, 'JobResponse', 'job_response_workflow')
    
    workflow.updateRoleMappings()
    
    print >> out, "Done installing workflows"
    
def addCatalogIndex(self, out, catalog, index, metaType, extra = None):
    """Add the given index name, of the given type, to the catalog."""
    
    if index not in catalog.indexes():
        catalog.addIndex(index, metaType, extra)
        catalog.reindexIndex(index, self.REQUEST)
        print >> out, "Added index", index, "to catalog"
    else:
        print >> out, "Index", index, "already in catalog"

def addCatalogMetadata(self, out, catalog, column):
    """Add the given column to the catalog's metadata schema"""
    
    if column not in catalog.schema():
        catalog.addColumn(column)
        print >> out, "Added", column, "to catalog metadata"
    else:
        print >> out, column, "already in catalog metadata"

def removeCatalogIndex(self, out, catalog, index):
    """Delete the given index"""
    
    if index in catalog.indexes():
        catalog.delIndex(index)
        print >> out, "Removed index", index
    else:
        print >> out, "Index", index, "not in catalog"
    
def removeCatalogMetadata(self, out, catalog, column):
    """Delete the given metadata column"""
    
    if column in catalog.schema():
        catalog.delColumn(column)
        print >> out, "Removed column", column
    else:
        print >> out, "Column", column, "not in catalog"

def addFormControllerAction(self, out, controller, template, status, 
                                contentType, button, actionType, action):
    """Add the given action to the portalFormController"""
    
    controller.addFormAction(template, status, contentType, 
                                button, actionType, action)
    print >> out, "Added action %s to %s" % (action, template)

def removeFormControllerAction(self, out, controller, template, status, 
                                contentType, button):
    """Delete the given action from the portalFormController"""
    
    container = controller.actions
    
    try:
        container.delete(FormActionKey(template, status, contentType,
                                            button, controller))
        print >> out, "Removed action for %s from %s" % (button, template)
    except KeyError: pass

def addPortalFactoryType(self, out, factory, metaType):
    """Add the given type to the list of types used by PortalFactory"""
    
    types = factory.getFactoryTypes().keys()
    types.append(metaType)
    factory.manage_setPortalFactoryTypes(listOfTypeIds = types)
    
    print >> out, "Added %s to portal_factory" % (metaType,)

def addParentMetaTypeNotToQuery(self, out, props, metaType):
    
    types = tuple(props.getProperty('parentMetaTypesNotToQuery'))
    if metaType not in types:
        types += (metaType,)
        props.manage_changeProperties(parentMetaTypesNotToQuery = types)

def install(self):

    out = StringIO()

    installTypes(self, out, listTypes(PROJECTNAME), PROJECTNAME)
    install_subskin(self, out, product_globals)
    installWorkflows(self, out)

    catalog    = getToolByName(self, 'portal_catalog')
    controller = getToolByName(self, 'portal_form_controller')
    factory    = getToolByName(self, 'portal_factory')
    properties = getToolByName(self, 'portal_properties')

    addCatalogIndex(self, out, catalog, 'selectedBoards', 'KeywordIndex')
    addCatalogMetadata(self, out, catalog, 'UID')

    addFormControllerAction(self, out,
                             controller, 
                             template = 'validate_integrity',
                             status = 'success',
                             contentType = 'JobResponse',
                             button = '',
                             actionType = 'traverse_to',
                             action = 'string:jobresponse_post') 
                             
    addPortalFactoryType(self, out, factory, 'JobCategory')
    addPortalFactoryType(self, out, factory, 'JobSubCategory')
    addPortalFactoryType(self, out, factory, 'JobBoard')
    addPortalFactoryType(self, out, factory, 'JobPosting')
    addPortalFactoryType(self, out, factory, 'JobResponse')
    
    navtree_properties = getattr(properties, 'navtree_properties')
    
    addParentMetaTypeNotToQuery(self, out, navtree_properties, 'JobPosting')
    
    return out.getvalue()
    
def uninstall(self):

    out = StringIO()
    
    catalog    = getToolByName(self, 'portal_catalog')
    controller = getToolByName(self, 'portal_form_controller')     
    
    removeCatalogIndex(self, out, catalog, 'selectedBoards')
    # UID is too general - other products may have added it
    
    removeFormControllerAction(self, out, 
                                controller, 
                                template = 'validate_integrity', 
                                status ='success',
                                contentType = 'JobResponse',
                                button = '')
    
    return out.getvalue()
