from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition

from Products.PloneJobBoard.permissions import *

def setupJob_board_workflow(wf):
    "..."
    wf.setProperties(title='Job board workflow')

    for s in ['inactive', 'active']:
        wf.states.addState(s)
    for t in ['activate', 'deactivate']:
        wf.transitions.addTransition(t)
    for v in ['action', 'review_history', 'actor', 'comments', 'time']:
        wf.variables.addVariable(v)
    for l in []:
        wf.worklists.addWorklist(l)
    for p in (AccessContentsInformation, View, ModifyPortalContent, ListFolderContents):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('active')

    ## States initialization
    sdef = wf.states['inactive']
    sdef.setProperties(title="""Inactive""",
                       transitions=('activate',))
    sdef.setPermission(AccessContentsInformation, 0, ['Manager', 'Owner'])
    sdef.setPermission(View, 0, ['Manager', 'Owner'])
    sdef.setPermission(ModifyPortalContent, 0, ['Manager', 'Owner'])
    sdef.setPermission(ListFolderContents, 0, ['Manager', 'Owner'])

    sdef = wf.states['active']
    sdef.setProperties(title="""Active""",
                       transitions=('deactivate',))
    sdef.setPermission(AccessContentsInformation, 1, ['Anonymous', 'Manager', 'Member', 'Owner'])
    sdef.setPermission(View, 1, ['Anonymous', 'Authenticated', 'Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission(ModifyPortalContent, 0, ['Manager', 'Owner'])
    sdef.setPermission(ListFolderContents, 0, ['Anonymous', 'Authenticated', 'Manager', 'Member', 'Owner', 'Reviewer'])

    ## Transitions initialization
    tdef = wf.transitions['deactivate']
    tdef.setProperties(title="""Member deactivates job board""",
                       new_state_id="""inactive""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""De-activate""",
                       actbox_url="""""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner; Manager'},
                       )

    tdef = wf.transitions['activate']
    tdef.setProperties(title="""Member activates job board""",
                       new_state_id="""active""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Activate""",
                       actbox_url="""""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner; Manager'},
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    ## Worklists Initialization

def createJob_board_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupJob_board_workflow(ob)
    return ob


addWorkflowFactory(createJob_board_workflow,
                   id='job_board_workflow',
                   title='Job board workflow')

    