from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition

from Products.PloneJobBoard.permissions import *

def setupJob_posting_workflow(wf):
    "..."
    wf.setProperties(title='Job posting workflow')

    for s in ['unpublished', 'published']:
        wf.states.addState(s)
    for t in ['retract', 'publish']:
        wf.transitions.addTransition(t)
    for v in ['action', 'review_history', 'actor', 'comments', 'time']:
        wf.variables.addVariable(v)
    for l in []:
        wf.worklists.addWorklist(l)
    for p in (AccessContentsInformation, View, ModifyPortalContent, 
                ListFolderContents, AddPortalContent, AddResponses):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('unpublished')

    ## States initialization
    sdef = wf.states['unpublished']
    sdef.setProperties(title="""Not yet published""",
                       transitions=('publish',))
    sdef.setPermission(AccessContentsInformation, 0, ['Manager', 'Owner'])
    sdef.setPermission(View, 0, ['Manager', 'Owner'])
    sdef.setPermission(ModifyPortalContent, 0, ['Manager', 'Owner'])
    sdef.setPermission(AddResponses, 0, [])
    sdef.setPermission(AddPortalContent, 0, [])
    sdef.setPermission(ListFolderContents, 0, ['Manager', 'Owner'])

    sdef = wf.states['published']
    sdef.setProperties(title="""Public""",
                       transitions=('retract',))
    sdef.setPermission(AccessContentsInformation, 1, ['Anonymous', 'Authenticated', 'Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission(View, 1, ['Anonymous', 'Authenticated', 'Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission(ModifyPortalContent, 0, [])
    sdef.setPermission(AddResponses, 0, ['Manager', 'Member', 'Reviewer'])
    sdef.setPermission(AddPortalContent, 0, ['Manager', 'Member', 'Reviewer'])
    sdef.setPermission(ListFolderContents, 0, ['Manager', 'Owner'])

    ## Transitions initialization
    tdef = wf.transitions['retract']
    tdef.setProperties(title="""Member retracts submission""",
                       new_state_id="""unpublished""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""retract_job_posting""",
                       actbox_name="""Retract""",
                       actbox_url="""%(content_url)s/content_retract_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner; Manager'},
                       )

    tdef = wf.transitions['publish']
    tdef.setProperties(title="""Poster publishes posting to board""",
                       new_state_id="""published""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""post_job_posting""",
                       actbox_name="""Publish""",
                       actbox_url="""%(content_url)s/content_publish_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner; Manager'},
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    ## Worklists Initialization

def createJob_posting_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupJob_posting_workflow(ob)
    return ob


addWorkflowFactory(createJob_posting_workflow,
                   id='job_posting_workflow',
                   title='Job posting workflow')

    