from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition

from Products.PloneJobBoard.permissions import *

def setupJob_response_workflow(wf):
    "..."
    wf.setProperties(title='Job response workflow')

    for s in ['creating', 'posted']:
        wf.states.addState(s)
    for t in ['post']:
        wf.transitions.addTransition(t)
    for v in ['action', 'review_history', 'actor', 'comments', 'time']:
        wf.variables.addVariable(v)
    for l in []:
        wf.worklists.addWorklist(l)
    for p in (AccessContentsInformation, View, ModifyPortalContent):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('creating')

    ## States initialization
    sdef = wf.states['creating']
    sdef.setProperties(title="""Creating""",
                       transitions=('post',))
    sdef.setPermission(AccessContentsInformation, 0, ['Manager', 'Owner'])
    sdef.setPermission(View, 0, ['Manager', 'Owner'])
    sdef.setPermission(ModifyPortalContent, 0, ['Manager', 'Owner'])

    sdef = wf.states['posted']
    sdef.setProperties(title="""Posted""",
                       transitions=())
    sdef.setPermission(AccessContentsInformation, 0, ['Manager', 'Owner'])
    sdef.setPermission(View, 0, ['Manager', 'Owner'])
    sdef.setPermission(ModifyPortalContent, 0, ['Manager', 'Owner'])


    ## Transitions initialization
    tdef = wf.transitions['post']
    tdef.setProperties(title="""""",
                       new_state_id="""posted""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""post_job_response""",
                       actbox_name="""Create""",
                       actbox_url="""""",
                       actbox_category="""workflow""",
                       props=None,
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    ## Worklists Initialization

def createJob_response_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupJob_response_workflow(ob)
    return ob
    
addWorkflowFactory(createJob_response_workflow,
                   id='job_response_workflow',
                   title='Job response workflow')

    