__author__  = 'Martin Aspeli <optilude@gmx.net>'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import *

from Products.PloneJobBoard import permissions

from security import JobBoardPermissionsMixin, SecuritySchema

JobBoardSchema = BaseSchema + SecuritySchema + Schema((
    StringField('title',
        accessor = 'Title',
        required = True,
        widget = StringWidget(
            description="""Please enter a title for the job board""",
            description_msgid='PloneJobBoard_help_jobboard_title',
            label="Title",
            label_msgid='PloneJobBoard_label_jobboard_title',
            i18n_domain='PloneJobBoard',
        ),
    ),
    
    TextField('description',
        accessor = "Description",
        required = False,
        widget = TextAreaWidget(
            description = """Please enter a short description for the job board""",
            description_msgid = 'PloneJobBoard_help_jobboard_description',
            label = """Description""",
            label_msgid = 'PloneJobBoard_label_jobboard_description',
            i18n_domain = 'PloneJobBoard',
        ),
    ),
    
))

class JobBoard(BaseContent, JobBoardPermissionsMixin):
    """Represents a job board, which must be contained inside a
    JobCategory or JobSubCategory. Members can create a JobPosting
    object and reference it to one or more JobBoard. Published job
    postings will then show up in the referenced board.
    """
    security = ClassSecurityInfo()
    portal_type = meta_type = 'JobBoard' 
    archetype_name = 'Job board'
    
    allow_discussion = False
    content_icon = 'JobBoard.gif'
    global_allow = False

    _at_rename_after_creation = True

    actions =  (
       {'action':      "string:${object_url}",
        'category':    "object",
        'id':          'view',
        'name':        'View',
        'permissions': (permissions.View,),
        'condition'  : 'python:1'
       },
       {'action':      "string:${object_url}/edit",
        'category':    "object",
        'id':          'edit',
        'name':        'Edit',
        'permissions': (permissions.ModifyPortalContent,),
        'condition'  : 'python:1'
       },
    )

    aliases = {
        '(Default)' : 'jobboard_view',
        'view'      : 'jobboard_view',
        'edit'      : 'base_edit'
    }
    
    schema = JobBoardSchema
    
    security.declareProtected(permissions.View, 'getPostings')
    def getPostings(self):
        """Get a catalogue query result set of the job postings in this
        category.
        """
        catalog = getToolByName(self, 'portal_catalog')
        return catalog.searchResults(portal_type  = 'JobPosting',
                                      review_state = 'published',
                                      selectedBoards = self.UID(),
                                      sort_on = 'Date',
                                      sort_order = 'desc')
                                      
registerType(JobBoard)


