__author__  = 'Martin Aspeli <optilude@gmx.net>'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *

from security import JobBoardPermissionsMixin, SecuritySchema

from Products.PloneJobBoard import permissions

JobCategorySchema = BaseFolderSchema + SecuritySchema + Schema((

    StringField('title',
        accessor = "Title",
        required = True,
        widget = StringWidget(
            description = """Please enter a title for this job board folder""",
            description_msgid = 'PloneJobBoard_help_category_title',
            i18n_domain = 'PloneJobBoard',
            label = "Title",
            label_msgid = 'PloneJobBoard_label_category_title',
        ),
    ),
    
    TextField('description',
        accessor = "Description",
        required = False,
        widget = TextAreaWidget(
            description = """Please enter a short description for this category of job boards""",
            description_msgid = 'PloneJobBoard_help_category_description',
            i18n_domain = 'PloneJobBoard',
            label = "Description",
            label_msgid = 'PloneJobBoard_label_category_description',
        ),
    ),
    
    BooleanField('required',
        index = ":schema",
        widget = BooleanWidget(
            description = """Whether or not new job postings must include a board in this category""",
            description_msgid = 'PloneJobBoard_help_category_required',
            i18n_domain = 'PloneJobBoard',
            label = "Required",
            label_msgid = 'PloneJobBoard_label_category_required',
        ),
    ),
    
    BooleanField('multiple',
        index = ":schema",
        default = True,
        widget = BooleanWidget(
            description = """Whether or not multiple boards in this category may be selected for a job posting""",
            description_msgid = 'PloneJobBoard_help_category_multiple',
            i18n_domain = 'PloneJobBoard',
            label = "Allow multiple",
            label_msgid = 'PloneJobBoard_label_category_multiple',
        ),
    ),
))

JobSubCategorySchema = BaseFolderSchema + SecuritySchema + Schema((
    StringField('title',
        accessor = 'Title',
        required = True,
        widget = StringWidget(
            description = """Please enter a title for the sub-category""",
            description_msgid = 'PloneJobBoard_help_subcategory_title',
            i18n_domain = 'PloneJobBoard',
            label = "Title",
            label_msgid = 'PloneJobBoard_label_title',
        ),
    ),
    
    TextField('description',
        accessor = 'Description',
        required = False,
        widget = TextAreaWidget(
            description = """Please enter a short description for this job board sub-category""",
            description_msgid = 'PloneJobBoard_help_description',
            i18n_domain = 'PloneJobBoard',
            label = "Description",
            label_msgid = 'PloneJobBoard_label_subcategory_description',
        ),
    ),
    
))

class JobCategory(BaseFolder, JobBoardPermissionsMixin):
    """A folder which may contain JobBoards, as well as JobSubCategory's. 
    This allows the creation of a hierarchy of categorised job boards. When 
    creating a job posting, one or more JobBoards may be specified on which 
    the posting will be displayed. If a category is 'required', a board from 
    the category (or a sub-category) must be selected for each job
    posting. If 'multiple' is false, only one job board in the
    category (including sub-categories) may be selected for a new
    job posting, otherwise multiple job boards may be selected.
    """
    security = ClassSecurityInfo()
    
    portal_type = meta_type = 'JobCategory' 
    archetype_name = 'Job category'
    allowed_content_types = ['JobBoard', 'JobSubCategory']
    filter_content_type = True
    allow_discussion = False
    content_icon = 'JobCategory.gif'
    global_allow = True
    
    _at_rename_after_creation = True
    
    actions =  (
       {'action':      "string:${object_url}",
        'category':    "object",
        'id':          'view',
        'name':        'View',
        'permissions': (permissions.View,),
        'condition'  : 'python:1'
       },
       {'action':      "string:${object_url}/edit",
        'category':    "object",
        'id':          'edit',
        'name':        'Edit',
        'permissions': (permissions.ModifyPortalContent,),
        'condition'  : 'python:1'
       },
    )
    
    aliases = {
        '(Default)' : 'jobcategory_view',
        'view'      : 'jobcategory_view',
        'edit'      : 'base_edit',
    }
    
    schema = JobCategorySchema

registerType(JobCategory)

class JobSubCategory(BaseFolder, JobBoardPermissionsMixin):
    """A sub-category, contained within a JobCategory. Sub categories
    may be used to organise job boards, but the 'required' and
    'multiple' attributes apply only to top-level categories.
    """
    security = ClassSecurityInfo()
    portal_type = meta_type = 'JobSubCategory' 
    archetype_name = 'Category'
    allowed_content_types = ('JobBoard', 'JobSubCategory',)
    filter_content_types = True
    allow_discussion = False
    content_icon = 'JobSubCategory.gif'
    global_allow = False
    
    _at_rename_after_creation = True
    
    actions =  (
       {'action':      "string:${object_url}",
        'category':    "object",
        'id':          'view',
        'name':        'View',
        'permissions': (permissions.View,),
        'condition'  : 'python:1'
       },
       {'action':      "string:${object_url}/edit",
        'category':    "object",
        'id':          'edit',
        'name':        'Edit',
        'permissions': (permissions.ModifyPortalContent,),
        'condition'  : 'python:1'
       },
    )
    
    aliases = {
        '(Default)' : 'jobsubcategory_view',
        'view'      : 'jobsubcategory_view',
        'edit'      : 'base_edit',
    }
    
    schema = JobSubCategorySchema

registerType(JobSubCategory)

