__author__  = 'Martin Aspeli <optilude@gmx.net>'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import *
from Products.Archetypes import transaction

from Products.PloneJobBoard import permissions

JobResponseSchema = BaseSchema  + Schema((
    ComputedField('title',
        accessor = 'Title',
        expression = '"%s %s, %s" %(context.getSalutation(), context.getSurname(), context.getGivenNames())',
        mode = 'r',
        widget = ComputedWidget(
            description = "The title of the response object, computed from the saluation, title and given names of the respondent.",
            description_msgid = 'PloneJobBoard_help_title',
            i18n_domain = 'PloneJobBoard',
            label = "Item title",
            label_msgid = 'PloneJobBoard_label_title',
            modes = ('view',),
            visible = {'edit' : 'invisible',
                       'view' : 'visible' },
        ),
    ),
    
    StringField('salutation',
        widget = StringWidget(
            description = """Salutation(e.g. Mr., Ms.)""",
            description_msgid = 'PloneJobBoard_help_salutation',
            i18n_domain = 'PloneJobBoard',
            label = "Salutation",
            label_msgid = 'PloneJobBoard_label_salutation',
        ),
    ),
    
    StringField('surname',
        required = True,
        widget = StringWidget(
            description = """Please enter your surname""",
            description_msgid = 'PloneJobBoard_help_surname',
            i18n_domain = 'PloneJobBoard',
            label = "Surname",
            label_msgid = 'PloneJobBoard_label_surname',
        ),
    ),
    
    StringField('givenNames',
        required = True,
        widget = StringWidget(
            description = """Please enter your given name(s)""",
            description_msgid = 'PloneJobBoard_help_givenNames',
            i18n_domain = 'PloneJobBoard',
            label = "Given name(s)",
            label_msgid = 'PloneJobBoard_label_givenNames',
        ),
    ),
    
    StringField('email',
        required = True,
        validators = ('isEmail',),
        widget = StringWidget(
            description = """Please enter a contact email address""",
            description_msgid = 'PloneJobBoard_help_email',
            i18n_domain = 'PloneJobBoard',
            label = "Email",
            label_msgid = 'PloneJobBoard_label_email',
        ),
    ),
    
    StringField('daytimePhone',
        widget = StringWidget(
            description = """Please enter a daytime telephone number""",
            description_msgid = 'PloneJobBoard_help_daytimePhone',
            i18n_domain = 'PloneJobBoard',
            label = "Daytime telephone number",
            label_msgid = 'PloneJobBoard_label_daytimePhone',
        ),
    ),
    
    StringField('eveningPhone',
        widget = StringWidget(
            description = """Please enter an evening telphone number""",
            description_msgid = 'PloneJobBoard_help_eveningPhone',
            i18n_domain = 'PloneJobBoard',
            label = "Evening telphone number",
            label_msgid = 'PloneJobBoard_label_eveningPhone',
        ),
    ),
    
    TextField('address',
        widget = TextAreaWidget(
            description = """Please enter your postal address""",
            description_msgid = 'PloneJobBoard_help_address',
            i18n_domain = 'PloneJobBoard',
            label = "Address",
            label_msgid = 'PloneJobBoard_label_address',
            rows = 6,
        ),
    ),
    
    TextField('description',
        accessor = 'Description',
        widget = TextAreaWidget(
            description = """Please enter a short(two or three lines) response in support of your resume. Use this an opportunity to draw attention to your resume.""",
            description_msgid = 'PloneJobBoard_help_description',
            i18n_domain = 'PloneJobBoard',
            label = "Response",
            label_msgid = 'PloneJobBoard_label_description',
        ),
    ),
    
    FileField('resume',
        storage = AttributeStorage(),
        widget = FileWidget(
            description = """Please upload your resume""",
            description_msgid = 'PloneJobBoard_help_resume',
            i18n_domain = 'PloneJobBoard',
            label = "Resume",
            label_msgid = 'PloneJobBoard_label_resume',
        ),
    ),
    
))

class JobResponse(BaseContent):
    """A JobResponse is stored inside a JobPosting, created by
    applicants to the job. Its principal field is the resume file
    field, but basic contact details and a brief supporting
    statement(used as the item's Description) is recorded with each
    response. Responses may also link to portal members' profiles
    when made by a logged-in user.
    """
    security = ClassSecurityInfo()
    portal_type = meta_type = 'JobResponse' 
    archetype_name = 'Response'
    allow_discussion = 0
    content_icon = 'JobResponse.gif'
    global_allow = False

    default_view = 'jobresponse_view'
    immediate_view = 'base_edit'

    _at_rename_after_creation = True

    actions =  (
       {'action':      "string:${object_url}",
        'category':    "object",
        'id':          'view',
        'name':        'View',
        'permissions': (permissions.View,),
        'condition'  : 'python:1'
       },
       {'action':      "string:${object_url}/edit",
        'category':    "object",
        'id':          'edit',
        'name':        'Edit',
        'permissions': (permissions.ModifyPortalContent,),
        'condition'  : 'python:1'
       },
    )

    aliases = {
        '(Default)'  : 'jobresponse_view',
        'view'       : 'jobresponse_view',
        'edit'       : 'base_edit',
    }

    schema = JobResponseSchema

    def _renameAfterCreation(self, check_auto_id=False):
        parent = self.aq_inner.aq_parent
        maxId = 0
        for id in parent.objectIds():
            try:
                intId = int(id)
                maxId = max(maxId, intId)
            except (TypeError, ValueError):
                pass
        newId = str(maxId + 1)
        # Can't rename without a subtransaction commit when using portal_factory!
        transaction.savepoint(optimistic=True)
        self.setId(newId)        

registerType(JobResponse)


