from AccessControl import ClassSecurityInfo

from Products.Archetypes.Field import ReferenceField
from Products.CMFCore.utils import getToolByName
from Products.PloneJobBoard.widgets import JobBoardsWidget

class JobBoardsField(ReferenceField):
    """A field for referencing job boards
    """

    __implements__ = ReferenceField.__implements__

    _properties = ReferenceField._properties.copy()
    _properties.update({
        'widget'      : JobBoardsWidget,
        'validators'  : ('isValidJobBoardSelection',),
        'multiValued' : True,
        })

    security  = ClassSecurityInfo()

    security.declarePublic('getJobBoardCategories')
    def getJobBoardCategories(self, context, unrestricted=False):
        """Get a list of dicts, each containing:
        
            - item      -> a catalog brain representing the category or board
            - children  -> a list of dicts for children of this node
            
        Only those categories or boards where the current member is allowed to
        post will be returned.
        """
        
        membership = getToolByName(context, 'portal_membership')
        member = membership.getAuthenticatedMember()
        memberRoles = member.getRolesInContext(context)
        
        itemsByPath = {}
        dummyRoot = '_root_'
        
        catalog = getToolByName(context, 'portal_catalog')
        results = catalog.searchResults(portal_type = ('JobBoard', 'JobCategory', 'JobSubCategory',),
                                        review_state = 'active',
                                        sort_order = 'getObjPositionInParent',)
                                       
        def getParentPath(node):
            if node.portal_type == 'JobCategory':
                return dummyRoot
            else:
                return '/'.join(node.getPath().split('/')[:-1])
                                        
        def insertOrUpdateNode(node):
            nodePath = node.getPath()
            if itemsByPath.has_key(nodePath):
                itemsByPath[nodePath]['item'] = node
            else:
                itemsByPath[nodePath] = {'item' : node, 'children' : []}
                   
        def attachToParent(node):
            nodePath = node.getPath()
            parentPath = getParentPath(node)
            nodeItem = itemsByPath[nodePath]
            if itemsByPath.has_key(parentPath):
                itemsByPath[parentPath]['children'].append(nodeItem)
            else:
                itemsByPath[parentPath] = {'item' : None, 'children' : [nodeItem]}
            
        def parentIsPruned(node):
            nodePath = node.getPath()
            parentPath = getParentPath(node)
            if itemsByPath.has_key(parentPath):
                if itemsByPath[parentPath].has_key('_pruned'):
                    return True
            return False
                  
        def nodeIsAllowed(node):
            if unrestricted:
                return True
            permittedRoles = node.getPermittedRoles
            if permittedRoles:
                satisfied = False
                for role in permittedRoles:
                    if role in memberRoles:
                        satisfied = True
                        break
                return satisfied
            else:
                return True
                       
        def pruneNode(node):
            nodePath = node.getPath()
            itemsByPath[nodePath] = {'item' : None, 'children' : (), '_pruned' : True}
                       
        def insertItem(node):
            if not parentIsPruned(node):
                if nodeIsAllowed(node):
                    insertOrUpdateNode(node)
                    attachToParent(node)
                else:
                    pruneNode(node)
            
        for r in results:
            insertItem(r)
    
        rootItem = itemsByPath.get(dummyRoot, None)
        if rootItem is not None:
            return rootItem['children']
        else:
            return []