import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneJobBoard.tests import pjbtc

from AccessControl import Unauthorized

class TestJobCategory(pjbtc.PloneJobBoardTestCase):
    
    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.workflow = self.portal.portal_workflow
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.cat1 = self.portal.cat1
        
    def testRolesVocab(self):
        self.failUnless('Member' in self.cat1.getAvailableRoles())
        self.failUnless('Manager' in self.cat1.getAvailableRoles())
        
    def testEdit(self):
        self.cat1.setTitle('A title')
        self.cat1.setDescription('A description')
        self.cat1.setRequired(True)
        self.cat1.setMultiple(True)
        self.cat1.setPermittedRoles(('Member', 'Manager',))
        
        self.assertEqual(self.cat1.Title(), 'A title')
        self.assertEqual(self.cat1.Description(), 'A description')
        self.assertEqual(self.cat1.getRequired(), True)
        self.assertEqual(self.cat1.getMultiple(), True)
        self.assertEqual(self.cat1.getPermittedRoles(), ('Member', 'Manager',))
        
    def testCreateAsNonManager(self):
        self.setRoles(('Member',))
        self.assertRaises(Unauthorized, self.folder.invokeFactory, 'JobCategory', 'cat2')
        
    def testWorkflow(self):
        self.workflow.doActionFor(self.cat1, 'deactivate')
        self.assertEqual(self.workflow.getInfoFor(self.cat1, 'review_state'), 'inactive')
        self.workflow.doActionFor(self.cat1, 'activate')
        self.assertEqual(self.workflow.getInfoFor(self.cat1, 'review_state'), 'active')

class TestJobSubCategory(pjbtc.PloneJobBoardTestCase):
    
    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.workflow = self.portal.portal_workflow
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.cat1 = self.portal.cat1
        self.cat1.invokeFactory('JobSubCategory', 'cat11')
        self.cat11 = self.cat1.cat11
        
    def testRolesVocab(self):
        self.failUnless('Member' in self.cat11.getAvailableRoles())
        self.failUnless('Manager' in self.cat11.getAvailableRoles())
        
    def testEdit(self):
        self.cat11.setTitle('A title')
        self.cat11.setDescription('A description')
        self.cat11.setPermittedRoles(('Member', 'Manager',))
        
        self.assertEqual(self.cat11.Title(), 'A title')
        self.assertEqual(self.cat11.Description(), 'A description')
        self.assertEqual(self.cat11.getPermittedRoles(), ('Member', 'Manager',))
        
    def testCreateOutsideCategory(self):
        self.assertRaises(ValueError, self.portal.invokeFactory, 'JobSubCategory', 'cat2')
        
    def testWorkflow(self):
        self.workflow.doActionFor(self.cat11, 'deactivate')
        self.assertEqual(self.workflow.getInfoFor(self.cat11, 'review_state'), 'inactive')
        self.workflow.doActionFor(self.cat11, 'activate')
        self.assertEqual(self.workflow.getInfoFor(self.cat11, 'review_state'), 'active')

class TestJobBoard(pjbtc.PloneJobBoardTestCase):
    
    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.workflow = self.portal.portal_workflow
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.cat1 = self.portal.cat1
        self.cat1.invokeFactory('JobSubCategory', 'cat11')
        self.cat11 = self.cat1.cat11
        self.cat11.invokeFactory('JobBoard', 'b111')
        self.b111 = self.cat1.cat11.b111
        
    def testRolesVocab(self):
        self.failUnless('Member' in self.b111.getAvailableRoles())
        self.failUnless('Manager' in self.b111.getAvailableRoles())
        
    def testEdit(self):
        self.b111.setTitle('A title')
        self.b111.setDescription('A description')
        self.b111.setPermittedRoles(('Member', 'Manager',))
        
        self.assertEqual(self.b111.Title(), 'A title')
        self.assertEqual(self.b111.Description(), 'A description')
        self.assertEqual(self.b111.getPermittedRoles(), ('Member', 'Manager',))
        
    def testCreateOutsideCategory(self):
        self.assertRaises(ValueError, self.portal.invokeFactory, 'JobBoard', 'b2')
        
    def testCreateInsideCategory(self):
        try:
            self.cat1.invokeFactory('JobBoard', 'b2')
        except ValueError:
            self.fail()
        
    def testWorkflow(self):
        self.workflow.doActionFor(self.b111, 'deactivate')
        self.assertEqual(self.workflow.getInfoFor(self.b111, 'review_state'), 'inactive')
        self.workflow.doActionFor(self.b111, 'activate')
        self.assertEqual(self.workflow.getInfoFor(self.b111, 'review_state'), 'active')
    
    def testGetPostings(self):
        self.folder.invokeFactory('JobPosting', 'p1')
        self.folder.invokeFactory('JobPosting', 'p2')
        self.folder.invokeFactory('JobPosting', 'p3')
        self.folder.invokeFactory('JobPosting', 'p4')

        self.folder.p1.setJobBoards((self.b111.UID(),))
        self.folder.p2.setJobBoards((self.b111.UID(),))
        self.folder.p3.setJobBoards((self.b111.UID(),))
        
        self.workflow.doActionFor(self.folder.p1, 'publish')
        self.workflow.doActionFor(self.folder.p2, 'publish')
        self.workflow.doActionFor(self.folder.p4, 'publish')
        
        self.folder.p1.reindexObject()
        self.folder.p2.reindexObject()
        self.folder.p3.reindexObject()
        self.folder.p4.reindexObject()
        
        postings = self.b111.getPostings()
        self.assertEqual(len(postings), 2)
        
        ids = [i.getId for i in postings]
        self.failUnless('p1' in ids)
        self.failUnless('p2' in ids)
        self.failIf('p3' in ids)
        self.failIf('p4' in ids)
        
    
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestJobCategory))
    suite.addTest(makeSuite(TestJobSubCategory))
    suite.addTest(makeSuite(TestJobBoard))
    return suite

if __name__ == '__main__':
    framework()
