import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneJobBoard.tests import pjbtc

from DateTime import DateTime
from AccessControl import Unauthorized

class TestField(pjbtc.PloneJobBoardTestCase):
    
    def afterSetUp(self):
        self.workflow = self.portal.portal_workflow
        self.setRoles(('Manager',))
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.portal.cat1.invokeFactory('JobSubCategory', 'cat11')
        self.portal.cat1.invokeFactory('JobSubCategory', 'cat12')
        self.portal.cat1.invokeFactory('JobBoard', 'board13')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board111')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board112')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board113')
        self.portal.cat1.cat12.invokeFactory('JobBoard', 'board121')
        self.portal.cat1.cat12.invokeFactory('JobSubCategory', 'cat122')
        self.portal.invokeFactory('Folder', 'f1')
        self.portal.f1.invokeFactory('JobCategory', 'cat2')
        self.portal.f1.cat2.invokeFactory('JobBoard', 'board21')
        self.portal.f1.cat2.invokeFactory('JobBoard', 'board22')
        self.cat1 = self.portal.cat1
        self.cat2 = self.portal.f1.cat2
        self.setRoles(('Member',))
        self.folder.invokeFactory('JobPosting', 'p1')
        self.p1 = self.folder.p1
        self.field = self.p1.getField('jobBoards')
        
    def reindex(self):
        self.cat1.reindexObject()
        self.cat1.cat11.reindexObject()
        self.cat1.cat11.board111.reindexObject()
        self.cat1.cat11.board112.reindexObject()
        self.cat1.cat11.board113.reindexObject()
        self.cat1.cat12.reindexObject()
        self.cat1.cat12.board121.reindexObject()
        self.cat1.cat12.cat122.reindexObject()
        self.cat2.reindexObject()
        self.cat2.board21.reindexObject()
        self.cat2.board22.reindexObject()
        
    def testBasicStructure(self):
        cats = self.field.getJobBoardCategories(self.p1)
        
        # Root categories
        self.assertEqual(len(cats), 2)
        self.assertEqual(cats[0]['item'].getId, 'cat1')
        self.assertEqual(cats[1]['item'].getId, 'cat2')
        
        # Sub-categories of cat1
        self.assertEqual(len(cats[0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['item'].getId, 'cat11')
        self.assertEqual(cats[0]['children'][1]['item'].getId, 'cat12')
        self.assertEqual(cats[0]['children'][2]['item'].getId, 'board13')
        
        # Sub-categories of cat1/cat11
        self.assertEqual(len(cats[0]['children'][0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['children'][0]['item'].getId, 'board111')
        self.assertEqual(cats[0]['children'][0]['children'][1]['item'].getId, 'board112')
        self.assertEqual(cats[0]['children'][0]['children'][2]['item'].getId, 'board113')
        
        # Sub-categories of cat1/cat12
        self.assertEqual(len(cats[0]['children'][1]['children']), 2)
        self.assertEqual(cats[0]['children'][1]['children'][0]['item'].getId, 'board121')
        self.assertEqual(cats[0]['children'][1]['children'][1]['item'].getId, 'cat122')
        
        # Sub-categories of cat2
        self.assertEqual(len(cats[1]['children']), 2)
        self.assertEqual(cats[1]['children'][0]['item'].getId, 'board21')
        self.assertEqual(cats[1]['children'][1]['item'].getId, 'board22')
        
    def testNoCategories(self):
        self.portal._delObject('cat1')
        self.portal.f1._delObject('cat2')
        cats = self.field.getJobBoardCategories(self.p1)
        self.assertEqual(len(cats), 0)
        
    def testInactiveCategoryExcluded(self):
        self.workflow.doActionFor(self.cat2, 'deactivate')
        
        self.reindex()
        cats = self.field.getJobBoardCategories(self.p1)
        
        # Root categories
        self.assertEqual(len(cats), 1)
        self.assertEqual(cats[0]['item'].getId, 'cat1')
        
        # Sub-categories of cat1
        self.assertEqual(len(cats[0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['item'].getId, 'cat11')
        self.assertEqual(cats[0]['children'][1]['item'].getId, 'cat12')
        self.assertEqual(cats[0]['children'][2]['item'].getId, 'board13')
        
        # Sub-categories of cat1/cat11
        self.assertEqual(len(cats[0]['children'][0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['children'][0]['item'].getId, 'board111')
        self.assertEqual(cats[0]['children'][0]['children'][1]['item'].getId, 'board112')
        self.assertEqual(cats[0]['children'][0]['children'][2]['item'].getId, 'board113')
        
        # Sub-categories of cat1/cat12
        self.assertEqual(len(cats[0]['children'][1]['children']), 2)
        self.assertEqual(cats[0]['children'][1]['children'][0]['item'].getId, 'board121')
        self.assertEqual(cats[0]['children'][1]['children'][1]['item'].getId, 'cat122')
        
    def testInactiveSubcategoryExcluded(self):
        self.workflow.doActionFor(self.cat1.cat11, 'deactivate')
        
        self.reindex()
        cats = self.field.getJobBoardCategories(self.p1)
        
        # Root categories
        self.assertEqual(len(cats), 2)
        self.assertEqual(cats[0]['item'].getId, 'cat1')
        self.assertEqual(cats[1]['item'].getId, 'cat2')
        
        # Sub-categories of cat1
        self.assertEqual(len(cats[0]['children']), 2)
        self.assertEqual(cats[0]['children'][0]['item'].getId, 'cat12')
        self.assertEqual(cats[0]['children'][1]['item'].getId, 'board13')
        
        # Sub-categories of cat1/cat12
        self.assertEqual(len(cats[0]['children'][0]['children']), 2)
        self.assertEqual(cats[0]['children'][0]['children'][0]['item'].getId, 'board121')
        self.assertEqual(cats[0]['children'][0]['children'][1]['item'].getId, 'cat122')
        
        # Sub-categories of cat2
        self.assertEqual(len(cats[1]['children']), 2)
        self.assertEqual(cats[1]['children'][0]['item'].getId, 'board21')
        self.assertEqual(cats[1]['children'][1]['item'].getId, 'board22')
        
    def testInactiveBoardExcluded(self):
        self.workflow.doActionFor(self.cat1.board13, 'deactivate')
        
        self.reindex()
        cats = self.field.getJobBoardCategories(self.p1)
        
        # Root categories
        self.assertEqual(len(cats), 2)
        self.assertEqual(cats[0]['item'].getId, 'cat1')
        self.assertEqual(cats[1]['item'].getId, 'cat2')
        
        # Sub-categories of cat1
        self.assertEqual(len(cats[0]['children']), 2)
        self.assertEqual(cats[0]['children'][0]['item'].getId, 'cat11')
        self.assertEqual(cats[0]['children'][1]['item'].getId, 'cat12')
        
        # Sub-categories of cat1/cat11
        self.assertEqual(len(cats[0]['children'][0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['children'][0]['item'].getId, 'board111')
        self.assertEqual(cats[0]['children'][0]['children'][1]['item'].getId, 'board112')
        self.assertEqual(cats[0]['children'][0]['children'][2]['item'].getId, 'board113')
        
        # Sub-categories of cat1/cat12
        self.assertEqual(len(cats[0]['children'][1]['children']), 2)
        self.assertEqual(cats[0]['children'][1]['children'][0]['item'].getId, 'board121')
        self.assertEqual(cats[0]['children'][1]['children'][1]['item'].getId, 'cat122')
        
        # Sub-categories of cat2
        self.assertEqual(len(cats[1]['children']), 2)
        self.assertEqual(cats[1]['children'][0]['item'].getId, 'board21')
        self.assertEqual(cats[1]['children'][1]['item'].getId, 'board22')
        
    def testRoleExclusions(self):
        self.cat1.setPermittedRoles(('Manager', 'Reviewer',))
        self.cat2.setPermittedRoles(('Member', 'Manager',))
        self.cat2.board21.setPermittedRoles(('Reviewer',))
        
        self.reindex()
        cats = self.field.getJobBoardCategories(self.p1)
        
        # Root categories
        self.assertEqual(len(cats), 1)
        self.assertEqual(cats[0]['item'].getId, 'cat2')
        # Sub-categories of cat2
        self.assertEqual(len(cats[0]['children']), 1)
        self.assertEqual(cats[0]['children'][0]['item'].getId, 'board22')

    def testRoleExclusionsUnrestrictedMode(self):
        self.cat1.setPermittedRoles(('Manager', 'Reviewer',))
        self.cat2.setPermittedRoles(('Member', 'Manager',))
        self.cat2.board21.setPermittedRoles(('Reviewer',))
        
        self.reindex()
        cats = self.field.getJobBoardCategories(self.p1, unrestricted=True)
        
        # Root categories
        self.assertEqual(len(cats), 2)
        self.assertEqual(cats[0]['item'].getId, 'cat1')
        self.assertEqual(cats[1]['item'].getId, 'cat2')
        
        # Sub-categories of cat1
        self.assertEqual(len(cats[0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['item'].getId, 'cat11')
        self.assertEqual(cats[0]['children'][1]['item'].getId, 'cat12')
        self.assertEqual(cats[0]['children'][2]['item'].getId, 'board13')
        
        # Sub-categories of cat1/cat11
        self.assertEqual(len(cats[0]['children'][0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['children'][0]['item'].getId, 'board111')
        self.assertEqual(cats[0]['children'][0]['children'][1]['item'].getId, 'board112')
        self.assertEqual(cats[0]['children'][0]['children'][2]['item'].getId, 'board113')
        
        # Sub-categories of cat1/cat12
        self.assertEqual(len(cats[0]['children'][1]['children']), 2)
        self.assertEqual(cats[0]['children'][1]['children'][0]['item'].getId, 'board121')
        self.assertEqual(cats[0]['children'][1]['children'][1]['item'].getId, 'cat122')
        
        # Sub-categories of cat2
        self.assertEqual(len(cats[1]['children']), 2)
        self.assertEqual(cats[1]['children'][0]['item'].getId, 'board21')
        self.assertEqual(cats[1]['children'][1]['item'].getId, 'board22')

    def testUnrestrictedModeStillIgnoresInactive(self):
        self.workflow.doActionFor(self.cat2, 'deactivate')
        
        self.reindex()
        cats = self.field.getJobBoardCategories(self.p1, unrestricted=True)
        
        # Root categories
        self.assertEqual(len(cats), 1)
        self.assertEqual(cats[0]['item'].getId, 'cat1')
        
        # Sub-categories of cat1
        self.assertEqual(len(cats[0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['item'].getId, 'cat11')
        self.assertEqual(cats[0]['children'][1]['item'].getId, 'cat12')
        self.assertEqual(cats[0]['children'][2]['item'].getId, 'board13')
        
        # Sub-categories of cat1/cat11
        self.assertEqual(len(cats[0]['children'][0]['children']), 3)
        self.assertEqual(cats[0]['children'][0]['children'][0]['item'].getId, 'board111')
        self.assertEqual(cats[0]['children'][0]['children'][1]['item'].getId, 'board112')
        self.assertEqual(cats[0]['children'][0]['children'][2]['item'].getId, 'board113')
        
        # Sub-categories of cat1/cat12
        self.assertEqual(len(cats[0]['children'][1]['children']), 2)
        self.assertEqual(cats[0]['children'][1]['children'][0]['item'].getId, 'board121')
        self.assertEqual(cats[0]['children'][1]['children'][1]['item'].getId, 'cat122')
        
    def testRoleExclusionsWithMorePermissiveChildren(self):
        self.cat1.setPermittedRoles(('Manager', 'Reviewer',))
        self.cat1.cat11.setPermittedRoles(('Member',))
        
        self.reindex()
        cats = self.field.getJobBoardCategories(self.p1)
        
        # Root categories
        self.assertEqual(len(cats), 1)
        self.assertEqual(cats[0]['item'].getId, 'cat2')
        
        # Sub-categories of cat2
        self.assertEqual(len(cats[0]['children']), 2)
        self.assertEqual(cats[0]['children'][0]['item'].getId, 'board21')
        self.assertEqual(cats[0]['children'][1]['item'].getId, 'board22')
        
    def testNoAvailableCategories(self):
        self.workflow.doActionFor(self.cat1, 'deactivate')
        self.workflow.doActionFor(self.cat2, 'deactivate')
        self.reindex()
        cats = self.field.getJobBoardCategories(self.p1)
        self.assertEqual(len(cats), 0)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestField))
    return suite

if __name__ == '__main__':
    framework()
