import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneJobBoard.tests import pjbtc

from DateTime import DateTime
from AccessControl import Unauthorized

from Products.PloneTestCase.setup import default_user

class TestPosting(pjbtc.PloneJobBoardTestCase):
    
    def afterSetUp(self):
        self.workflow = self.portal.portal_workflow
        self.props = self.portal.portal_properties.site_properties
        
        self.setRoles(('Manager',))
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.cat1 = self.portal.cat1
        self.cat1.invokeFactory('JobBoard', 'b11')
        self.b11 = self.cat1.b11
        self.setRoles(('Member',))
        
        self.folder.invokeFactory('JobPosting', 'p1')
        self.p1 = self.folder.p1
        
    def testEdit(self):
        self.p1.setTitle('A title')
        self.p1.setDescription('A description')
        self.p1.setJobType('A job type')
        self.p1.setEmploymentType('An employment type')
        self.p1.setJobLocation('A job location')
        self.p1.setJobDescription('<p>A job description</p>')
        self.p1.setJobRequirements(('One', 'Two'))
        self.p1.setContactName('A contact name')
        self.p1.setContactEmail('name@company.com')
        self.p1.setContactPhone('+44 020 123 4567')
        self.p1.setContactAddress('An address\nOn two lines')
        self.p1.setExtraInformation('Some extra information')
        self.p1.setJobBoards((self.b11.UID(),))
        self.p1.setActiveDays(10)
        
        self.assertEqual(self.p1.Title(), 'A title')
        self.assertEqual(self.p1.Description(), 'A description')
        self.assertEqual(self.p1.getJobType(), 'A job type')
        self.assertEqual(self.p1.getEmploymentType(), 'An employment type')
        self.assertEqual(self.p1.getJobLocation(), 'A job location')
        self.assertEqual(self.p1.getJobDescription(), '<p>A job description</p>')
        self.assertEqual(self.p1.getJobRequirements(), ('One', 'Two',))
        self.assertEqual(self.p1.getContactName(), 'A contact name')
        self.assertEqual(self.p1.getContactEmail(), 'name@company.com')
        self.assertEqual(self.p1.getContactPhone(), '+44 020 123 4567')
        self.assertEqual(self.p1.getContactAddress(), 'An address\nOn two lines')
        self.assertEqual(self.p1.getExtraInformation(), 'Some extra information')
        self.assertEqual(self.p1.getRawJobBoards(), [self.b11.UID()])
        self.assertEqual(self.p1.selectedBoards(), [self.b11.UID()])
        self.assertEqual(self.p1.getActiveDays(), 10)
        
    def testWorkflow(self):
        self.workflow.doActionFor(self.p1, 'publish')
        self.assertEqual(self.workflow.getInfoFor(self.p1, 'review_state'), 'published')
        self.workflow.doActionFor(self.p1, 'retract')
        self.assertEqual(self.workflow.getInfoFor(self.p1, 'review_state'), 'unpublished')

    def testExpirationDate(self):
        self.p1.setActiveDays(10)
        self.assertEquals(self.p1.getExpirationDate(), None)
        self.workflow.doActionFor(self.p1, 'publish')
        self.assertEquals(self.p1.getExpirationDate(), self.p1.getEffectiveDate() + 10)

    def testGetActiveDaysDefault(self):
        self.assertEqual(self.p1.getActiveDaysDefault(), 30)
        self.props.manage_addProperty('jobPostingDefaultDays', 60, 'int')
        self.assertEqual(self.p1.getActiveDaysDefault(), 60)

    def testValidateActiveDays(self):
        self.assertEqual(self.p1.validate_activeDays('foo'), "You must enter a positive whole number")
        self.assertEqual(self.p1.validate_activeDays(30), None)
        self.assertEqual(self.p1.validate_activeDays(31), "You may not make a job posting active for more than %s days" % (30))
        self.props.manage_addProperty('jobPostingDefaultDays', 60, 'int')
        self.assertEqual(self.p1.validate_activeDays(31), None)
        self.assertEqual(self.p1.validate_activeDays(61), "You may not make a job posting active for more than %s days" % (60))
        self.props.manage_changeProperties(jobPostingDefaultDays = 0)
        self.assertEqual(self.p1.validate_activeDays(200), None)
        
    def testIsOpenWorkflow(self):
        self.assertEqual(self.p1.isOpen(), False)
        self.workflow.doActionFor(self.p1, 'publish')
        self.assertEqual(self.p1.isOpen(), True)
    
    def testIsOpenExpiration(self):
        self.workflow.doActionFor(self.p1, 'publish')
        self.p1.setEffectiveDate(DateTime(1980, 1, 1))
        self.p1.setExpirationDate(DateTime(1981, 1, 1))
        self.assertEqual(self.p1.isOpen(), False)
        
    def testCanRespondWorkflow(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        
        self.login('member2')
        self.assertEqual(self.p1.canRespond(), False)
        
        self.login(default_user)
        self.workflow.doActionFor(self.p1, 'publish')
        
        self.login('member2')
        self.assertEqual(self.p1.canRespond(), True)
        
    def testCanRespondExpiration(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        self.workflow.doActionFor(self.p1, 'publish')
        self.p1.setEffectiveDate(DateTime(1980, 1, 1))
        self.p1.setExpirationDate(DateTime(1981, 1, 1))
        
        self.login('member2')
        self.assertEqual(self.p1.canRespond(), False)
        
    def testCanRespondOwner(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        self.workflow.doActionFor(self.p1, 'publish')
        self.assertEqual(self.p1.canRespond(), False)
        self.assertEqual(self.p1.isOwner(), True)
        self.login('member2')
        self.assertEqual(self.p1.canRespond(), True)
        self.assertEqual(self.p1.isOwner(), False)
        
    def testCanRespondAlreadyResponded(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        self.workflow.doActionFor(self.p1, 'publish')
        self.login('member2')
        self.assertEqual(self.p1.canRespond(), True)
        self.assertEqual(self.p1.alreadyResponded(), False)
        self.p1.invokeFactory('JobResponse', '1')
        self.workflow.doActionFor(self.p1['1'], 'post')
        self.p1['1'].reindexObject()
        self.assertEqual(self.p1.canRespond(), False)
        self.assertEqual(self.p1.alreadyResponded(), True)
        
    def testCanRespondAnon(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        self.workflow.doActionFor(self.p1, 'publish')
        self.login('member2')
        self.assertEqual(self.p1.canRespond(), True)
        self.logout()
        self.assertEqual(self.p1.canRespond(), False)
        
    def testGetNotAddableTypesWorkflow(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        
        self.login('member2')
        self.assertEqual(self.p1.getNotAddableTypes(), ('JobResponse',))
        
        self.login(default_user)
        self.workflow.doActionFor(self.p1, 'publish')
        
        self.login('member2')
        self.assertEqual(self.p1.getNotAddableTypes(), ())
        
    def testGetNotAddableTypesExpiration(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        self.workflow.doActionFor(self.p1, 'publish')
        self.p1.setEffectiveDate(DateTime(1980, 1, 1))
        self.p1.setExpirationDate(DateTime(1981, 1, 1))
        
        self.login('member2')
        self.assertEqual(self.p1.getNotAddableTypes(), ('JobResponse',))
        
    def testGetNotAddableTypesOwner(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        self.workflow.doActionFor(self.p1, 'publish')
        self.assertEqual(self.p1.getNotAddableTypes(), ('JobResponse',))
        self.login('member2')
        self.assertEqual(self.p1.getNotAddableTypes(), ())
        
    def testGetNotAddableTypesAlreadyResponded(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        self.workflow.doActionFor(self.p1, 'publish')
        self.login('member2')
        self.assertEqual(self.p1.getNotAddableTypes(), ())
        self.p1.invokeFactory('JobResponse', '1')
        self.workflow.doActionFor(self.p1['1'], 'post')
        self.p1['1'].reindexObject()
        self.assertEqual(self.p1.getNotAddableTypes(), ('JobResponse',))
        

class TestResponse(pjbtc.PloneJobBoardTestCase):

    def afterSetUp(self):
        self.addMember('member2', 'Member2', 'member2@member.com', ('Member',), DateTime())
        self.workflow = self.portal.portal_workflow
        
        self.setRoles(('Manager',))
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.cat1 = self.portal.cat1
        self.cat1.invokeFactory('JobBoard', 'b11')
        self.b11 = self.cat1.b11
        self.setRoles(('Member',))
        
        self.folder.invokeFactory('JobPosting', 'p1')
        self.p1 = self.folder.p1
        self.workflow.doActionFor(self.p1, 'publish')
        
        self.login('member2')
        self.p1.invokeFactory('JobResponse', 'r11')
        self.r11 = self.p1.r11
        self.workflow.doActionFor(self.r11, 'post')
        self.login(default_user)
        
        
    def testEdit(self):
        self.r11.setDescription('A description')
        self.r11.setSalutation('Mr.')
        self.r11.setSurname('Jones')
        self.r11.setGivenNames('David John')
        self.r11.setEmail('name@company.com')
        self.r11.setDaytimePhone('+44 020 123 456')
        self.r11.setEveningPhone('+44 020 789 789')
        self.r11.setAddress('An address\nOn two lines')
        
        self.assertEqual(self.r11.Description(), 'A description')
        self.assertEqual(self.r11.getSalutation(), 'Mr.')
        self.assertEqual(self.r11.getSurname(), 'Jones')
        self.assertEqual(self.r11.getGivenNames(), 'David John')
        self.assertEqual(self.r11.Title(), 'Mr. Jones, David John')
        self.assertEqual(self.r11.getEmail(), 'name@company.com')
        self.assertEqual(self.r11.getDaytimePhone(), '+44 020 123 456')
        self.assertEqual(self.r11.getEveningPhone(), '+44 020 789 789')
        self.assertEqual(self.r11.getAddress(), 'An address\nOn two lines')
    
    def testPostedOwner(self):
        self.assertEqual(self.r11.owner_info()['id'], default_user)
        self.assertEqual(self.r11.Creator(), 'member2')
        
    def testRenameAfterCreation(self):
        self.r11._renameAfterCreation()
        self.assertEqual(self.r11.getId(), '1')
        self.p1.invokeFactory('JobResponse', 'foo')
        self.p1.foo._renameAfterCreation()
        self.assertEqual(self.p1.objectIds(), ['1', '2'])
        
    def testCreateOutsideBoard(self):
        self.assertRaises(ValueError, self.folder.invokeFactory, 'JobResponse', 'r2')
        
    def testCreateInUnpublishedPosting(self):
        self.folder.invokeFactory('JobPosting', 'p2')
        self.assertRaises(Unauthorized, self.folder.p2.invokeFactory, 'JobResponse', 'r2')
    
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPosting))
    suite.addTest(makeSuite(TestResponse))
    return suite

if __name__ == '__main__':
    framework()
