import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneJobBoard.tests import pjbtc


class TestInstallation(pjbtc.PloneJobBoardTestCase):
    """Ensure product is properly installed"""

    def afterSetUp(self):
        self.skins           = self.portal.portal_skins
        self.types           = self.portal.portal_types
        self.factory         = self.portal.portal_factory
        self.catalog         = self.portal.portal_catalog
        self.workflow        = self.portal.portal_workflow
        self.properties      = self.portal.portal_properties
        self.transforms      = self.portal.portal_transforms
        self.form_controller = self.portal.portal_form_controller

        self.pjbTypes = {'JobCategory'    : 'job_board_workflow',
                         'JobSubCategory' : 'job_board_workflow',
                         'JobBoard'       : 'job_board_workflow',
                         'JobPosting'     : 'job_posting_workflow',
                         'JobResponse'    : 'job_response_workflow'}

    def testPJBInstalled(self):
        self.failUnless('PloneJobBoard' in self.skins.objectIds())
        for t in self.pjbTypes.keys():
            self.failUnless(t in self.types.objectIds())

    def testJobPostingInParentMetaTypesNotToQuery(self):
        self.failUnless('JobPosting' in self.properties.navtree_properties.parentMetaTypesNotToQuery)

    def testWorkflowsInstalled(self):
        for k, v in self.pjbTypes.items():
            self.failUnless(v in self.workflow.objectIds())
            self.failUnless(self.workflow.getChainForPortalType(k) == (v,))

    def testPortalFactorySetup(self):
        for t in self.pjbTypes.keys():
            self.failUnless(t in self.factory.getFactoryTypes())

    def testCatalogIndexInstalled(self):
        self.failUnless('selectedBoards' in self.catalog.indexes())

    def testCatalogMetadataInstalled(self):
        self.failUnless('getJobType' in self.catalog.schema())
        self.failUnless('getJobLocation' in self.catalog.schema())
        self.failUnless('getRequired' in self.catalog.schema())
        self.failUnless('getMultiple' in self.catalog.schema())
        self.failUnless('getPermittedRoles' in self.catalog.schema())
        self.failUnless('UID' in self.catalog.schema())

class TestContentCreation(pjbtc.PloneJobBoardTestCase):
    """Ensure content types can be created"""

    def afterSetUp(self):
        self.workflow = self.portal.portal_workflow

    def testCreateCategory(self):
        self.setRoles(('Manager',))
        self.portal.invokeFactory('JobCategory', 'cat1')

    def testCreateSubCategory(self):
        self.setRoles(('Manager',))
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.portal.cat1.invokeFactory('JobSubCategory', 'subcat1')
    
    def testCreateBoard(self):
        self.setRoles(('Manager',))
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.portal.cat1.invokeFactory('JobSubCategory', 'subcat1')
        self.portal.cat1.subcat1.invokeFactory('JobBoard', 'board1')

    def testCreatePosting(self):
        self.folder.invokeFactory('JobPosting', 'posting1')

    def testCreateResponse(self):
        self.folder.invokeFactory('JobPosting', 'posting1')
        self.workflow.doActionFor(self.folder.posting1, 'publish')
        self.folder.posting1.invokeFactory('JobResponse', 'response1')

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestContentCreation))
    return suite

if __name__ == '__main__':
    framework()
