import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneJobBoard.tests import pjbtc

from Products.PloneJobBoard.validators import JobBoardSelectionValidator

class TestValidator(pjbtc.PloneJobBoardTestCase):

    def afterSetUp(self):
        self.workflow = self.portal.portal_workflow
        self.setRoles(('Manager',))
        self.portal.invokeFactory('JobCategory', 'cat1')
        self.portal.cat1.invokeFactory('JobSubCategory', 'cat11')
        self.portal.cat1.invokeFactory('JobSubCategory', 'cat12')
        self.portal.cat1.invokeFactory('JobBoard', 'board13')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board111')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board112')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board113')
        self.portal.cat1.cat12.invokeFactory('JobBoard', 'board121')
        self.portal.cat1.cat12.invokeFactory('JobSubCategory', 'cat122')
        self.portal.invokeFactory('Folder', 'f1')
        self.portal.f1.invokeFactory('JobCategory', 'cat2')
        self.portal.f1.cat2.invokeFactory('JobBoard', 'board21')
        self.portal.f1.cat2.invokeFactory('JobBoard', 'board22')
        self.cat1 = self.portal.cat1
        self.cat2 = self.portal.f1.cat2
        self.setRoles(('Member',))
        self.folder.invokeFactory('JobPosting', 'p1')
        self.p1 = self.folder.p1
        self.field = self.p1.getField('jobBoards')
        self.validator = JobBoardSelectionValidator('testValidator')

    def reindex(self):
        self.cat1.reindexObject()
        self.cat1.cat11.reindexObject()
        self.cat1.cat11.board111.reindexObject()
        self.cat1.cat11.board112.reindexObject()
        self.cat1.cat11.board113.reindexObject()
        self.cat1.cat12.reindexObject()
        self.cat1.cat12.board121.reindexObject()
        self.cat1.cat12.cat122.reindexObject()
        self.cat2.reindexObject()
        self.cat2.board21.reindexObject()
        self.cat2.board22.reindexObject()

    def validate(self):
        value = self.p1.getRawJobBoards()
        return self.validator(value, field=self.field, instance=self.p1)

    def testUnpermittedCategory(self):
        self.cat1.setPermittedRoles(('Manager', 'Reviewer',))
        self.reindex()
        self.p1.setJobBoards((self.cat2.board21, self.cat1.cat11.board111,))
        self.failUnless("The job board 'board111' is no longer available to you." in self.validate())
        self.failIf("The job board 'board21' is no longer available to you" in self.validate())

    def testInactiveCategory(self):
        self.workflow.doActionFor(self.cat1, 'deactivate')
        self.reindex()
        self.p1.setJobBoards((self.cat2.board21, self.cat1.cat11.board111,))
        self.failUnless("The job board 'board111' is no longer available to you." in self.validate())
        self.failIf("The job board 'board21' is no longer available to you" in self.validate())
        
    def testInactiveSubCategory(self):
        self.workflow.doActionFor(self.cat1.cat11, 'deactivate')
        self.reindex()
        self.p1.setJobBoards((self.cat2.board21, self.cat1.cat11.board111,))
        self.failUnless("The job board 'board111' is no longer available to you." in self.validate())
        self.failIf("The job board 'board21' is no longer available to you" in self.validate())
    
    def testInactiveBoard(self):
        self.workflow.doActionFor(self.cat1.cat11.board111, 'deactivate')
        self.reindex()
        self.p1.setJobBoards((self.cat2.board21, self.cat1.cat11.board111,))
        self.failUnless("The job board 'board111' is no longer available to you." in self.validate())
        self.failIf("The job board 'board21' is no longer available to you" in self.validate())
        
    def testRequiredCategory(self):
        self.cat2.setRequired(True)
        self.reindex()
        self.failUnless('You must select a board in category' in self.validate())
        self.p1.setJobBoards((self.cat2.board21,))
        self.assertEqual(self.validate(), None)
        
    def testRequiredCategoryComplexSelection(self):
        self.cat1.setRequired(True)
        self.reindex()
        self.failUnless('You must select a board in category' in self.validate())
        self.p1.setJobBoards((self.cat2.board21, self.cat1.cat11.board111,))
        self.assertEqual(self.validate(), None)
        
    def testSingleSelectCategory(self):
        self.cat2.setMultiple(False)
        self.reindex()
        self.assertEqual(self.validate(), None)
        self.p1.setJobBoards((self.cat2.board21, self.cat1.cat11.board111,))
        self.assertEqual(self.validate(), None)
        self.p1.setJobBoards((self.cat2.board21, self.cat2.board22, self.cat1.cat11.board111,))
        self.failUnless('You can only select one board in category' in self.validate())
        
    def testBogusUID(self):
        self.p1.setJobBoards((self.cat1.cat11.board111, self.cat2.board21,))
        self.cat1.cat11._delObject('board111')
        # Note: Archetypes handles stale references for us, no need to worry;
        # the warning message should never be printed, but it's nice to be safe.
        self.assertEqual(self.validate(), None)
    
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestValidator))
    return suite

if __name__ == '__main__':
    framework()
