import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneJobBoard.tests import pjbtc

from Products.PloneJobBoard.widgets import JobBoardsWidget

class TestWidget(pjbtc.PloneJobBoardTestCase):

    def afterSetUp(self):
        self.workflow = self.portal.portal_workflow
        self.setRoles(('Manager',))
        self.portal.invokeFactory('JobCategory', 'cat1', title = 'Cat 1', description = 'Category 1')
        self.portal.cat1.invokeFactory('JobSubCategory', 'cat11', title = 'Cat 11', description = 'Category 11')
        self.portal.cat1.invokeFactory('JobSubCategory', 'cat12', title = 'Cat 12', description = 'Category 12')
        self.portal.cat1.invokeFactory('JobBoard', 'board13', title = 'B 13', description = 'Board 13')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board111', title = 'B 111', description = 'Board 111')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board112', title = 'B 112', description = 'Board 112')
        self.portal.cat1.cat11.invokeFactory('JobBoard', 'board113', title = 'B 113', description = 'Board 113')
        self.portal.cat1.cat12.invokeFactory('JobBoard', 'board121', title = 'B 121', description = 'Board 121')
        self.portal.cat1.cat12.invokeFactory('JobSubCategory', 'cat122', title = 'Cat 122', description = 'Category 122')
        self.portal.invokeFactory('Folder', 'f1')
        self.portal.f1.invokeFactory('JobCategory', 'cat2', title = 'Cat 2', description = 'Category 2')
        self.portal.f1.cat2.invokeFactory('JobBoard', 'board21', title = 'B 21', description = 'Board 21')
        self.portal.f1.cat2.invokeFactory('JobBoard', 'board22', title = 'B 22', description = 'Board 22')
        self.cat1 = self.portal.cat1
        self.cat2 = self.portal.f1.cat2
        self.setRoles(('Member',))
        self.folder.invokeFactory('JobPosting', 'p1')
        self.p1 = self.folder.p1
        self.field = self.p1.getField('jobBoards')
        self.widget = self.field.widget

    def calculate(self):
        return self.widget.getCategories(self.field, self.p1)

    def selected(self):
        return self.widget.getSelectedBoardsByCategory(self.field, self.p1)

    def testStructure(self):
        c = self.calculate()
        
        self.assertEqual(len(c), 2)
        self.assertEqual(c[0]['title'], 'Cat 1')
        self.assertEqual(c[1]['title'], 'Cat 2')
        self.assertEqual(c[0]['description'], 'Category 1')
        self.assertEqual(c[1]['description'], 'Category 2')

        self.assertEqual(len(c[0]['boards']), 5)
        self.assertEqual(c[0]['boards'][0]['title'], 'Cat 11 // B 111')
        self.assertEqual(c[0]['boards'][1]['title'], 'Cat 11 // B 112')
        self.assertEqual(c[0]['boards'][2]['title'], 'Cat 11 // B 113')
        self.assertEqual(c[0]['boards'][3]['title'], 'Cat 12 // B 121')
        self.assertEqual(c[0]['boards'][4]['title'], 'B 13')
        
        self.assertEqual(c[0]['boards'][0]['uid'], self.cat1.cat11.board111.UID())
        self.assertEqual(c[0]['boards'][1]['uid'], self.cat1.cat11.board112.UID())
        self.assertEqual(c[0]['boards'][2]['uid'], self.cat1.cat11.board113.UID())
        self.assertEqual(c[0]['boards'][3]['uid'], self.cat1.cat12.board121.UID())
        self.assertEqual(c[0]['boards'][4]['uid'], self.cat1.board13.UID())
        
        self.assertEqual(c[0]['boards'][0]['url'], self.cat1.cat11.board111.absolute_url())
        self.assertEqual(c[0]['boards'][1]['url'], self.cat1.cat11.board112.absolute_url())
        self.assertEqual(c[0]['boards'][2]['url'], self.cat1.cat11.board113.absolute_url())
        self.assertEqual(c[0]['boards'][3]['url'], self.cat1.cat12.board121.absolute_url())
        self.assertEqual(c[0]['boards'][4]['url'], self.cat1.board13.absolute_url())
        
        self.assertEqual(len(c[1]['boards']), 2)
        
        self.assertEqual(c[1]['boards'][0]['title'], 'B 21')
        self.assertEqual(c[1]['boards'][1]['title'], 'B 22')
        
        self.assertEqual(c[1]['boards'][0]['uid'], self.cat2.board21.UID())
        self.assertEqual(c[1]['boards'][1]['uid'], self.cat2.board22.UID())
        
        self.assertEqual(c[1]['boards'][0]['url'], self.cat2.board21.absolute_url())
        self.assertEqual(c[1]['boards'][1]['url'], self.cat2.board22.absolute_url())
        
    def testDisabledCategory(self):
        self.workflow.doActionFor(self.cat1, 'deactivate')
        c = self.calculate()
        self.assertEqual(len(c), 1)
        self.assertEqual(len(c[0]['boards']), 2)
        self.assertEqual(c[0]['boards'][0]['title'], 'B 21')
        self.assertEqual(c[0]['boards'][1]['title'], 'B 22')
        self.assertEqual(c[0]['boards'][0]['uid'], self.cat2.board21.UID())
        self.assertEqual(c[0]['boards'][1]['uid'], self.cat2.board22.UID())
        
    def testNoCategories(self):
        self.workflow.doActionFor(self.cat1, 'deactivate')
        self.workflow.doActionFor(self.cat2, 'deactivate')
        c = self.calculate()
        self.assertEqual(len(c), 0)

    def testGetSelected(self):
        self.p1.setJobBoards((self.cat1.cat11.board111, self.cat1.board13, self.cat2.board21,))
        s = self.selected()
        self.assertEqual(len(s), 2)
        self.assertEqual(s[0]['title'], 'Cat 1')
        self.assertEqual(s[1]['title'], 'Cat 2')
        
        self.assertEqual(len(s[0]['boards']), 2)
        self.assertEqual(s[0]['boards'][0]['title'], 'Cat 11 // B 111')
        self.assertEqual(s[0]['boards'][1]['title'], 'B 13')
        
        self.assertEqual(len(s[1]['boards']), 1)
        self.assertEqual(s[1]['boards'][0]['title'], 'B 21')
        
    def testGetSelectedNoSelection(self):
        self.p1.setJobBoards(())
        s = self.selected()
        self.assertEqual(len(s), 2)
        self.assertEqual(s[0]['title'], 'Cat 1')
        self.assertEqual(s[1]['title'], 'Cat 2')
        self.assertEqual(len(s[0]['boards']), 0)
        self.assertEqual(len(s[1]['boards']), 0)
        
    def testGetSelectedIncludesUnpermittedBoards(self):
        self.cat1.setPermittedRoles(('Manager',))
        self.cat1.reindexObject()
        self.p1.setJobBoards((self.cat1.cat11.board111, self.cat1.board13, self.cat2.board21,))
        s = self.selected()
        self.assertEqual(len(s), 2)
        self.assertEqual(s[0]['title'], 'Cat 1')
        self.assertEqual(s[1]['title'], 'Cat 2')
        
        self.assertEqual(len(s[0]['boards']), 2)
        self.assertEqual(s[0]['boards'][0]['title'], 'Cat 11 // B 111')
        self.assertEqual(s[0]['boards'][1]['title'], 'B 13')
        
        self.assertEqual(len(s[1]['boards']), 1)
        self.assertEqual(s[1]['boards'][0]['title'], 'B 21')

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestWidget))
    return suite

if __name__ == '__main__':
    framework()
