from Products.CMFCore.utils import getToolByName

from Products.validation.config import validation
from Products.validation.interfaces.IValidator import IValidator

class JobBoardSelectionValidator:
    """Ensure the selected job boards are valid according to the settings on 
    the boards.
    """

    __implements__ = IValidator

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kw):
        
        field   = kw['field']
        context = kw['instance']
        
        categories = field.getJobBoardCategories(context)

        processed = {}
        errorMessages = []
        
        uids = {}
        for v in value:
            uids[v] = 1
        
        def findSelectionInCategory(category):
            selection = []
            node = category['item']
            if node.portal_type == 'JobBoard':
                boardUID = node.UID
                if boardUID in uids:
                    processed[boardUID] = 1
                    selection.append(boardUID)
            for c in category['children']:
                selection += findSelectionInCategory(c)
            return selection
        
        def validateCategory(category):
            node = category['item']
            
            title = node.Title
            required = node.getRequired
            multiple = node.getMultiple
        
            # Note: this has the side-effect of incrementing 'processed'
            selectionInCategory = findSelectionInCategory(category)
        
            if required and len(selectionInCategory) == 0:
                errorMessages.append("You must select a board in category '%s'" % (title,))
            elif not multiple and len(selectionInCategory) > 1:
                errorMessages.append("You can only select one board in category '%s'" % (title,))
                 
        for c in categories:
            validateCategory(c)
        
        if len(processed) != len(value):
            reference_catalog = getToolByName(context, 'reference_catalog')
            for v in value:
                if v not in processed:
                    obj = reference_catalog.lookupObject(v)
                    if obj is not None:
                        errorMessages.append("The job board '%s' is no longer available to you." % (obj.title_or_id(),))
                    else:
                        errorMessages.append("WARNING: The job board with UID '%s' no longer exists!" % (v,))
                         
        if len(errorMessages) > 0:
            return "; ".join(errorMessages)
        else:
            return None
        
        
validation.register(JobBoardSelectionValidator('isValidJobBoardSelection',
                                                title='Job board selection', 
                                                description='Ensure the job board selection meets the requirements sets in categories and sub-categories'))