from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName

from Products.Archetypes.Widget import ReferenceWidget
from Products.Archetypes.Registry import registerWidget

class JobBoardsWidget(ReferenceWidget):
    """Widget to display job board selector
    """

    __implements__ = ReferenceWidget.__implements__

    _properties = ReferenceWidget._properties.copy()
    _properties.update({ 'macro'             : 'widget_jobboardselector',
                         'categorySeparator' : ' // ',})
                
    security  = ClassSecurityInfo()

    security.declarePublic('getCategories')
    def getCategories(self, field, context, unrestricted=False):
        """Get the categories from the given field, and return a data structure
        better suited for display purposes. This will contain a list dicts:
        
            title       - the title of the category
            description - the description of the category
            required    - whether the category is required
            multiple    - whether the category allows multiple selections
            boards      - a list of boards in this category
        
        The list of boards contains all boards in the given category. The titles
        of boards will be prefixed by any parent sub-categories the board is
        classified under. The list contains dicts with the following keys:
        
            title   - the title of the board, prefixed with any sub-categories
            uid     - the board's UID
            url     - the board's URL
        """
    
        categories = []
        tree = field.getJobBoardCategories(context, unrestricted=unrestricted)
        
        separator = self.categorySeparator
        
        def flattenSubCategories(boards, category, prefix=None):
            for c in category['children']:
                node = c['item']
                title = prefix and prefix + separator + node.Title or node.Title
                if node.portal_type == 'JobBoard':
                    boards.append({ 'title' : title,
                                    'uid'   : node.UID,
                                    'url'   : node.getURL(),
                                  })
                else:
                    flattenSubCategories(boards, c, title)
            
        for category in tree:
            node = category['item']
            boards = []
            flattenSubCategories(boards, category)
            categories.append({ 'title'       : node.Title,
                                'description' : node.Description,
                                'required'    : node.getRequired,
                                'multiple'    : node.getMultiple,
                                'boards'      : boards,
                               })
        return categories
                
    security.declarePublic('getSelectedBoardsByCategory')
    def getSelectedBoardsByCategory(self, field, context):
        """Get a structure equivalent to that returned by getCategories(), but
        containing only selected job boards.
        """
        uids = field.getRaw(context)
        categories = self.getCategories(field, context, unrestricted=True)
        
        selected = []
        
        for c in categories:
            boards = []
            for b in c['boards']:
                if b['uid'] in uids:
                    boards.append(b)

            item = c.copy()
            item['boards'] = boards
            selected.append(item)
        
        return selected
                
registerWidget(JobBoardsWidget,
                title = 'Job board selector widget',
                description= ('Renders a HTML widget used to select job boards',),
                used_for = ('Products.PloneJobBoard.fields.JobBoardsField',)
                )
